"""
/* sudoku_ui.py - sudoku puzzle UI game
 *
 * Copyright 2007-2008 OpenMoko, Inc.
 * Authored by Erin Yueh <erin_yueh@openmoko.com>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 * Inspired by NMAccessPoint from libnm-glib.
 */
""" 

#!/usr/bin/python

import ecore.evas
import ecore
import edje
import random
#import etk

WINDOW_WIDTH = 455
WINDOW_HEIGHT = 640
GROUP_ROWCOL = {
                '0': (0,0,0), '1':(1,0,3), '2':(2,0,6),
                '3': (3,3,0), '4':(4,3,3), '5':(5,3,6),
                '6': (6,6,0), '7':(7,6,3), '8':(8,6,6),
               }
def checkRow(row,value):
    isOK = True
    puzzle = ee.data["puzzle"]
    #print 'checkRow',row,value
    for j in xrange(9):
        #print puzzle[row][j]
        if(value == puzzle[row][j]):
            #print 'checkRow', row,j,value
            return False
    return isOK
    
def checkColumn(column,value):
    isOK = True
    puzzle = ee.data["puzzle"]
    #print 'checkColumn',column,value
    for i in xrange(9):
        #print puzzle[i][column]
        if(value == puzzle[i][column]):
            #print 'checkColumn',i,column,value
            return False
    return isOK

def checkGroup(group,value):
    isOK = True
    puzzle = ee.data["puzzle"]
    #print 'checkGroup',group,value
    (group,r,c) = GROUP_ROWCOL[str(group)]
    for i in xrange (r,r+3):
        for j in xrange(c,c+3):
            #print puzzle[i][j]
            if(value == puzzle[i][j]):
                #print 'checkGroup',group,i,j,value
                return False
                #isOK = False
    return isOK

def solve():
    isOK = True
    puzzle = ee.data["puzzle"]
    row = [[],[],[],[],[],[],[],[],[]]
    column = [[],[],[],[],[],[],[],[],[]]
    group = [[],[],[],[],[],[],[],[],[]]
    for i in xrange (9):
        for j in xrange(9):
            value = puzzle[i][j]
            g = (i/3)*3 + (j/3)
            #print i,j,g,value
            row[i].append(value)
            column[j].append(value)
            group[g].append(value)
    # should only display once
    for x in xrange(9):
        for v in xrange(1,10):
            #print 'r,c,g,v: ',row[x].count(v), column[x].count(v), group[x].count(v),v
            if not(row[x].count(v) == 1 and column[x].count(v) ==1 and group[x].count(v) ==1):
                isOK = False

    return isOK

def analyze(button,value):
    isOK = True
    #puzzle = ee.data["puzzle"]
    addr = button[6:]
    #print addr, value
    i = int(addr) / 9
    j = int(addr) % 9
    g = (i/3)*3 + (j/3)
    #print i,j,g,addr,value
    if(checkRow(i,value) == False or checkColumn(j,value) == False or checkGroup(g,value) == False):
        return False

    return isOK
    
def number_key_cb(zoom_in, emission, source):
    #print 'number_key_cb emission,source:',emission,source
    action = ee.data["action_txt"]
    action.text_set("")
    if(source == 'clear' or source == 'create'):
        # init undo redo action track
        ee.data["undo_counter"] = 0
        ee.data["redo_counter"] = 0
        ee.data["undo_track"] = []
        ee.data["redo_track"] = []
        ee.data["clicking"] = None
        if(source == 'clear'):
            puzzle_clone = ee.data["puzzle_clone"]
            puzzle = ee.data["puzzle"]
            for i in xrange (9):
                for j in xrange(9):
                    value = puzzle_clone[i][j]
                    #print 'i,j,value,puzzle[i][j]',i,j,value,puzzle_clone[i][j]
                    puzzle[i][j] = value
                    addr = j + i *9
                    text_data = "button" + str(addr)
                    text = ee.data[text_data] 
                    text.text_set(str(value))
                    if(value>0):
                        text.color_set(50,50,50,180)
                    elif(value==0):
                        text.text_set("")
            #print 'clear all data!!'
        elif(source == 'create'):
            #print 'create new puzzle'
            createNewPuzzle()
    elif(source =='undo'):
        if(ee.data["undo_counter"] == 0):
            return
        #print ee.data["undo_counter"], ee.data["undo_track"]
        undo_counter = ee.data["undo_counter"]
        (addr,value) = ee.data["undo_track"][undo_counter-1]
        redo = ee.data["undo_track"].pop()
        ee.data["redo_track"].append(redo)
        redo_counter = ee.data["redo_counter"]
        ee.data["redo_counter"] = redo_counter+1
        i = int(addr) / 9
        j = int(addr) % 9
        original_value = ee.data["puzzle_clone"][i][j]
        text = ee.data["button"+str(addr)]
        #print i,j,original_value,text
        text.text_set("")
        ee.data["puzzle"][i][j] = original_value
        if(undo_counter>0):
            ee.data["undo_counter"] = undo_counter-1
    elif(source == 'redo'):
        if(ee.data["redo_counter"] == 0):
            return
        #print ee.data["redo_counter"], ee.data["redo_track"]
        redo_counter = ee.data["redo_counter"]
        (addr,value) = ee.data["redo_track"][redo_counter-1]
        undo = ee.data["redo_track"].pop()
        ee.data["undo_track"].append(undo)
        undo_counter = ee.data["undo_counter"]
        ee.data["undo_counter"] = undo_counter+1
        
        i = int(addr) / 9
        j = int(addr) % 9
        text = ee.data["button"+str(addr)]
        #print i,j,value,text
        text.text_set(str(value))
        text.color_set(0,0,255,255)
        ee.data["puzzle"][i][j] = int(value)
        if(redo_counter>0):
            ee.data["redo_counter"] = redo_counter-1
        
    elif(source == 'solve'):
        
        if(solve() == False):
            action.text_set("FAILED!!")
        else:
            action.text_set("Congratulations!")

    else: 
        # fill in key number
        if(ee.data["clicking"]):
            text = ee.data[ee.data["clicking"]]
            text.text_set(source)
            button = ee.data["clicking"]
            #addr = button[6:]
            addr = ee.data["clicking"][6:]
            i = int(addr) / 9
            j = int(addr) % 9
            #print i,j,addr
            
            if(analyze(ee.data["clicking"],int(source)) == False):
                text.color_set(255,0,0,255)
            else: 
                text.color_set(0,0,0,255)
            # real fill in the value to puzzle data
            puzzle = ee.data["puzzle"]
            puzzle[i][j] = int(source)
            # record the step
            undo_counter = ee.data["undo_counter"]
            ee.data["undo_counter"] = undo_counter +1
            step = (int(addr),int(source))
            ee.data["undo_track"].append(step)
            #print 'undo_counter:',ee.data["undo_counter"]
            #print 'undo_track:',ee.data["undo_track"]

    return True

def getButtonPos(source):
    addr = source[6:]
    i = int(addr) / 9
    j = int(addr) % 9
    #print i,j,addr
    
    x = 5 + j *50
    y = i *50
    
    return x,y

def puzzle_key_cb(zoom_in, emission, source):
    #print 'puzzle_key_cb emission,source:',emission,source
    
    text = ee.data[source]
    if(text.data["fixed"] == False):
        text.color_set(255,255,255,255)
        ee.data["clicking"] = source
        # display focus 
        focus = ee.data["focus"]
        focus.color_set(0,0,0,100)
        focus.stack_below(text)
        (x,y) = getButtonPos(source)
        focus.pos_set(x,y)
    return True

def get_puzzle():

        file_name = "/usr/share/pyefl-sudoku/puzzle/1000"
        fIn = open(file_name, 'r')
        fIn.seek(82*(random.randint(1, 1000)))
        l = fIn.readline()

        if (len(l) != 82):
            print "the puzzle is wrong!!"

        p = []
        for i in xrange(9):
            t = []
            for j in xrange(9):
                t.append(int(l[i * 9 + j]))
            p.append(t)

        return p

def createNewPuzzle():
    
    puzzle = get_puzzle()
    #print 'new puzzle',puzzle
    for i in xrange(9):
        for j in xrange(9):
            ee.data["puzzle"][i][j] = puzzle[i][j]
            ee.data["puzzle_clone"][i][j] = puzzle[i][j]
            value = puzzle[i][j]
            addr = j + i *9
            text_data = "button" + str(addr)
            text = ee.data[text_data] 
            text.text_set(str(value))
            if(value>0):
                text.color_set(50,50,50,180)
                text.data["fixed"] = True
            elif(value==0):
                text.data["fixed"] = False
                text.text_set("")

    return True

def resize_cb(ee):
    
    bg = ee.data["bg"]
    canvas = ee.evas
    bg.size = canvas.size
    area = ee.data["area"]
    area.size = canvas.size
    
    return True
    
def main():

    global ee

    ee = ecore.evas.SoftwareX11(w=455, h=640)
    ee.title_set('sudoku')
    ee.size_min_set(WINDOW_WIDTH,WINDOW_HEIGHT)
    ee.size_max_set(WINDOW_WIDTH,WINDOW_HEIGHT)
    canvas = ee.evas

    # create a background 
    bg = edje.Edje(canvas, file="/usr/share/pyefl-sudoku/sudoku.edj", group="pythonEFL-sudoku/background")
    bg.pos = (0, 0)
    bg.size = canvas.size
    bg.show()
    ee.data["bg"] = bg

    # create number keyboard and action icons
    area = edje.Edje(canvas, file="/usr/share/pyefl-sudoku/sudoku.edj", group="pythonEFL-sudoku/panel")
    area.signal_callback_add("mouse,clicked,*", "*", number_key_cb)
    area.pos = (0, 0)
    area.size = canvas.size
    area.show()
    ee.data["area"] = area

    # create puzzle board
    board = edje.Edje(canvas, file="/usr/share/pyefl-sudoku/sudoku.edj", group="pythonEFL-sudoku/board")
    board.signal_callback_add("mouse,clicked,*", "*", puzzle_key_cb)
    board.pos = (0, 0)
    board.show()

    # retrieve one puzzle data
    puzzle = get_puzzle()
    puzzle_clone = []
    for i in xrange(9):
        row = []
        for j in xrange(9):
            row.append(puzzle[i][j])
        puzzle_clone.append(row)
    #print 'puzzle',puzzle, puzzle_clone
    ee.data["puzzle"] = puzzle
    ee.data["puzzle_clone"] = puzzle_clone
    ee.data["clicking"] = None

    # init redo undo track
    ee.data["undo_counter"] = 0
    ee.data["redo_counter"] = 0
    ee.data["undo_track"] = []
    ee.data["redo_track"] = []
    
    element_w = 450 / 9
    element_h = 450 / 9 
    # put all elements to Text objects
    for i in xrange (9):
        for j in xrange(9):
            x = 15 + (element_w * (i))
            y = 15 + (element_h * (j))
            value = puzzle[j][i]
            addr = j *9 + i
            #print 'row,column,value,addr: ',i,j,value,addr
            text = canvas.Text(text=str(value), font=("sans serif", 20), color=(0,255,255,255))
            if(value>0):
                text.data["fixed"] = True
                text.color_set(50,50,50,180)
            else:
                text.data["fixed"] = False
                text.text_set("")
            text.pos_set(x,y)
            text.show()
            text_data = "button" + str(addr)
            ee.data[text_data] = text
    
    # display action
    action = canvas.Text(text="", font=("sans serif", 20), color=(200,0,0,155))
    action.pos_set(150,550)
    action.show()
    ee.data["action_txt"] = action
    
    # display a focus rectangle
    focus = canvas.Rectangle()
    focus.pos_set(5,0)
    focus.size_set(50,50)
    focus.color_set(0,0,0,0)
    ee.data["focus"] = focus
    focus.show()
    ee.callback_resize = resize_cb
    ee.show()
    #ecore.animator_add(animate_line)
    #ecore.animator_frametime_set(1.0 / 30.0)   
    ecore.main_loop_begin()
    
if __name__ == "__main__":
	main()